from PySide2.QtCore import QUrl, Qt
from PySide2.QtGui import QIcon, QDesktopServices
from PySide2.QtWidgets import QDesktopWidget, QMainWindow
from PySide2.QtWebEngineWidgets import QWebEngineView, QWebEnginePage


class Window:

    def __init__(self, url, title, icon=None):

        self.__zoom = 1.0
        self.__start_url = url
        self.window = QMainWindow()
        self.__webview = QWebEngineView()
        self.__webview.setContextMenuPolicy(Qt.NoContextMenu)
        self.window.setCentralWidget(self.__webview)

        self.window.setWindowTitle(title)
        if icon is not None:
            self.window.setWindowIcon(QIcon(icon))
        self.window.resize(QDesktopWidget().availableGeometry(self.window).size() * 0.75);

    def resize(self, width, height):
        self.window.resize(width, height)

    def set_zoom(self, value):
        value = float(value)
        self.__webview.setZoomFactor(self.zoom)

    def start(self):
        self.__webview.setPage(WebPage(self.__start_url, parent=self.__webview))
        self.window.show()

class WebPage(QWebEnginePage):

    def __init__(self, url, parent=None):
        super(WebPage, self).__init__(parent)
        self.__url = url
        self.load(QUrl(self.__url))

    def acceptNavigationRequest(self, qurl, navtype, mainframe):
        if self.__url in qurl.toString():  # open in QWebEngineView
            return True
        else:
            QDesktopServices.openUrl(qurl)
            return False


from PySide2.QtCore import QUrl, QCoreApplication, Qt, Slot, Signal, QObject
from PySide2.QtWidgets import QApplication, QFileDialog
from PySide2.QtWebEngineWidgets import QWebEngineView
from pathlib import Path
from threading import Event, Thread

from .window import Window
from .splash import SplashScreen


class Application(QObject):

    windows = {}

    folder_dialog_spawn = Signal(str)
    window_dialog_spawn = Signal(int, str, str, str)

    def __init__(self, name="Default app name"):
        QObject.__init__(self)

        self.__file_dialog_event = Event()
        self.__filename_dialog = None
        self.__splash = None

        self.folder_dialog_spawn.connect(self.__on_folder_dialog)
        self.window_dialog_spawn.connect(self.__on_window)
        QCoreApplication.setAttribute(Qt.AA_EnableHighDpiScaling, True)
        self.__app = QApplication([])
        self.__app.setApplicationName(name)

    def spawn_window(self, url="", title="Default title", icon=None):
        id = len(Application.windows)
        self.window_dialog_spawn.emit(id, url, title, icon)
        return id

    def spawn_folder_dialog(self, title="Select directory"):
        self.folder_dialog_spawn.emit(title)
        self.__file_dialog_event.wait()
        path = self.__filename_dialog
        self.__filename_dialog = None
        return Path(path) if path else None

    def add_splashscreen(self, path):
        self.__splash = SplashScreen(path)

    def add_flask(self, flask, host="127.0.0.1", port=0, debug=False, spawn_window=True):

        def start_flask(host, port, debug):
            self.__flask_app.run(host=host, port=port, debug=debug, threaded=True)

        import socket

        if port <= 1024:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.bind(('localhost', 0))
            port = sock.getsockname()[1]
            sock.close()

        self.__flask_app = flask
        self.__flask_thread = Thread(
            target=start_flask, args=(host, port, debug), daemon=True)

        url = "http://{}:{}".format(host, port)

        if spawn_window:
            return self.spawn_window(url)

    @Slot(int, str)
    def __on_window(self, id, url, title, icon):
        window = Window(url=url, title=title, icon=icon)
        Application.windows[id] = window

    @Slot(str)
    def __on_folder_dialog(self, title):
        self.__filename_dialog = str(QFileDialog.getExistingDirectory(
            parent=None, caption=title, options=QFileDialog.ShowDirsOnly))
        self.__file_dialog_event.set()

    def start(self):
        from sys import exit

        if self.__splash is not None:
            self.__splash.show()

            if len(Application.windows) > 0:
                self.__splash.close(Application.windows[0].window)

        try:
            self.__flask_thread.start()
        except AttributeError as e:
            pass

        for window in Application.windows.values():
            window.start()

        return exit(self.__app.exec_())



from PySide2.QtWidgets import QSplashScreen
from PySide2.QtGui import QPixmap, QMovie, QPainter

class SplashScreen:

    def __init__(self, path):
        self.__path = str(path)

        supported_formats = (format.data().decode() for format in QMovie.supportedFormats())
        if self.__path.lower().endswith(tuple(supported_formats)):
            self.splash = AnimatedSplash(path)
        else:
            self.splash = ImageSplash(path)

    def show(self, time=None):
        self.splash.show()
    
    def close(self, window):
        self.splash.finish(window)


class ImageSplash(QSplashScreen):

    def __init__(self, path):
        self.__pixmap = QPixmap(path)
        QSplashScreen.__init__(self, self.__pixmap)


class AnimatedSplash(QSplashScreen):

    def __init__(self, path):

        self.__movie = QMovie(path)
        self.__movie.jumpToFrame(0)

        pixmap = QPixmap(self.__movie.frameRect().size())
        QSplashScreen.__init__(self, pixmap)
        self.__movie.frameChanged.connect(self.repaint)

    def showEvent(self, event):
        self.__movie.start()

    def hideEvent(self, event):
        self.__movie.stop()

    def paintEvent(self, event):
        painter = QPainter(self)
        pixmap = self.__movie.currentPixmap()
        self.setMask(pixmap.mask())
        painter.drawPixmap(0, 0, pixmap)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pycuteweb",
    version="0.1.1",
    author="Matteo Meneghetti",
    author_email="matteo@meneghetti.dev",
    description="Package which aims to display webapps or websites on a desktop window",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/matteomeneghetti/pycuteweb",
    packages=setuptools.find_packages(),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3'
    ],
    install_requires=["pyside2"],
    python_requires='>=3.6'
)